## Introduction

Serial port over ethernet using a HTTP server with WebSockets and REST API.

`NOTE: This is an early version of the script - not yet ready for production environment. Please be patient.`

## Getting Started

Install the package:
```bash
npm install -g remote-serial-port-server
```

Start the server:
```bash
remote-serial-port-server --port 5147
```

Open in web browser:
```bash
http://localhost:5147/
```

## Features

* list and control serial ports remotely
* REST API
* WebSocket to serial port
* simple web interface
* monitor traffic
* shared serial port (connect multiple clients to a single serial port)

## Command Line

```
Usage:
  remote-serial-port-server [options]
  
Options:
  --help                 Print this message
  --list                 Print serial ports and exit
  --port, -p [num]       HTTP and WebSocket port number, default: 5147
  --no-list              Disable serial port list
  --no-read              Disable read ops
  --no-write             Disable write ops
  --no-ui                Disable web interface
  --no-ws                Disable web socket
  --allow-ports [list]   Allow only specific ports
  
Examples:
  remote-serial-port-server
  remote-serial-port-server --list
  remote-serial-port-server --no-read --no-write --no-ws --port 80
  remote-serial-port-server --allow-ports COM1,COM2,COM3
```

## REST API

See [REST API Documentation](API.md)

## WebSocket

From within a browser
```javascript
//Initialize a WebSocket connection
var ws = new WebSocket('ws://localhost:5147/api/v1/port/COM1/data');
ws.binaryType = 'arraybuffer';
ws.onopen = function(event) {
  console.log("WebSocket connected");
};
ws.onclose = function(event) {
  console.log("WebSocket disconnected");
};
ws.onmessage = function(event) {
  console.log("WebSocket read");
};
ws.onerror = function(event) {
  console.error("WebSocket error", error);
};

//Send something to serial port
ws.send("AT;\n");
```

## Client-side

See [remote-serial-port-client](https://github.com/papnkukn/remote-serial-port-client) library.

## Using with Express

```javascript
var express = require('express');
var app = express();

//Register the remote serial port REST API with the prefix
var srv = require('remote-serial-port-server');
app.use("/api/v1", srv.http({ verbose: true }));

var port = 5147;
var server = app.listen(port, function() {
  console.log('HTTP on port ' + server.address().port);
});
```

## Web Interface

http://localhost:5147/

![Web Interface Screenshot](/wiki/screenshot-web-interface.png)

## Usage Scenarios

1. Accessing a serial port via internet, lets say port 80 and web interface. Could be set up with a secure connection using SSL for both HTTP and WebSocket using a proxy, e.g. nginx or Apache.
2. Sharing a single serial port with multiple clients. Connected clients can read/write to one serial port. User can also monitor the traffic for development and debugging purposes.
3. Accessing a serial port on virtual machines when there is no hardware attached.
4. Also used for crappy serial port drivers for cheap chinese Arduino clones. This can be done by running a virtual machine and attaching a USB device. Drivers are then installed to the virtual machine instead of host machine.
5. Using a Raspberry Pi as a remote serial port host, e.g. hosting a WS2300 weather station connected to a Raspberry Pi and controlled from a desktop computer.
6. Access to a serial port from node-webkit or electron UI frames.